# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""re-add Queued capability state

Revision ID: 53cc148d8e1f
Revises: d65a2ee9cc3a
Create Date: 2021-10-12 14:58:30.557126

"""
from alembic import op

# revision identifiers, used by Alembic.
revision = "53cc148d8e1f"
down_revision = "d65a2ee9cc3a"
branch_labels = None
depends_on = None


def add_queued_state():
    sql = """
    UPDATE capability_state_transitions SET (pattern, to_state) =
    ('event_type == capability-submitted', 'Queued')
    WHERE from_state = 'Start';

    INSERT INTO capability_state_transitions (capability_name, from_state, pattern, to_state)
    SELECT capability_name, 'Queued', 'event_type == workflow-executing', 'Executing' FROM capabilities
    WHERE capability_steps ~ E'^prepare-and-run-workflow (.*)\nawait-workflow$';
    """
    op.execute(sql)


def remove_queued_state():
    sql = """
    UPDATE capability_state_transitions SET (pattern, to_state) = ('event_type == capability-submitted', 'Executing')
    WHERE from_state = 'Start';

    DELETE FROM capability_state_transitions WHERE to_state = 'Queued';
    """
    op.execute(sql)


def upgrade():
    add_queued_state()


def downgrade():
    remove_queued_state()
