#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Creates messages needed for capabilities """

# pylint: disable=E0401

import logging

from immutable_views import DictView

from workspaces.shared_interfaces import MessageArchitectIF

logger = logging.getLogger(__name__)

"""
Class which creates an immutable dictionary of amqp message templates for easy access
"""

# DictView(Dict[msg_type, msg_content]
service_templates = DictView(
    {
        "capability": {
            "service": "capability",
            "routing_key": "capability",
            "subject": None,
        }
    }
)

capability_msg_templates = DictView(
    {
        "capability_submitted": {
            "type": "capability-submitted",
        },
        "capability_complete": {
            "type": "capability-complete",
        },
        "capability_failed": {
            "type": "capability-failed",
        },
        "execution_failed": {
            "type": "execution-failed",
        },
        "execution_error": {
            "type": "execution-error",
        },
        "execution_complete": {
            "type": "execution-complete",
        },
        "restart_queue": {
            "type": "restart-queue",
        },
    }
)


class CapabilityMessageArchitect(MessageArchitectIF):
    """Creates messages needed used by capabilities"""

    def __init__(
        self,
        execution: "CapabilityExecution" = None,
        request: "CapabilityRequest" = None,
        version: "CapabilityVersion" = None,
    ):
        self.subject = execution or request or version

    @staticmethod
    def get_message_template(msg_type: str) -> dict:
        return {**service_templates["capability"], **capability_msg_templates[msg_type]}

    def compose_message(self, msg_type: str) -> DictView:
        template = self.get_message_template(msg_type)
        template = template.copy()
        template["subject"] = self.subject
        return DictView(template)
